﻿using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Simbir.Market.Account.Attributes;
using Simbir.Market.Account.Controllers.DTO;
using Simbir.Market.Account.Exceptions;
using Simbir.Market.Account.Services;
using Simbir.Market.Account.Services.DTO;
using System.ComponentModel.DataAnnotations;
using System.Security.Claims;

namespace Simbir.Market.Account.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class AccountsController : ControllerBase
    {
        private readonly IMapper _mapper;
        private readonly IUserService _userService;

        public AccountsController(IMapper mapper, IUserService userService)
        {
            _mapper = mapper;
            _userService = userService;
        }

        [HttpGet("Me")]
        [Authorize]
        public async Task<IActionResult> GetMe()
        {
            if (!long.TryParse(User.FindFirstValue(ClaimTypes.NameIdentifier), out var userId))
            {
                throw new ApiException("Информация о пользователе отсутствует.");
            }

            var userInfo = await _userService.GetMeInfoByIdAsync(userId);

            return Ok(_mapper.Map<AccountMeInfoResponse>(userInfo));
        }

        [HttpPut("Update")]
        [Authorize]
        public async Task<IActionResult> Update([FromBody] AccountUpdateRequest userUpdateDTO)
        {
            if (!long.TryParse(User.FindFirstValue(ClaimTypes.NameIdentifier), out var userId))
            {
                throw new ApiException("Информация о пользователе отсутствует.");
            }

            var updateDTO = _mapper.Map<UserUpdateDTO>(userUpdateDTO);

            await _userService.UpdateAsync(userId, updateDTO);

            return Ok();
        }

        [HttpGet]
        [Authorize("Admin")]
        public async Task<IActionResult> GetAll([FromQuery] int from = 0, [FromQuery] int count = 0)
        {
            var list = await _userService.GetAllAsync(from, count);

            return Ok(list);
        }

        [HttpPost]
        [Authorize("Admin")]
        public async Task<IActionResult> CreateUser([FromBody] SignUpUserRequest request)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            var userDTO = _mapper.Map<UserCreateByAdminDTO>(request);

            var userId = await _userService.CreateByAdminAsync(userDTO);

            return Ok(userId);
        }

        [HttpPut("{id:long}")]
        [Authorize("Admin")]
        public async Task<IActionResult> UpdateUser([Required] long id, [FromBody] UpdateUserRequest request)
        {
            var userDTO = _mapper.Map<UserUpdateByAdminDTO>(request);

            await _userService.UpdateUserAsync(id, userDTO);

            return Ok();
        }

        [HttpDelete("{id:long}")]
        [Authorize("Admin")]
        public async Task<IActionResult> DeleteUser([Required] long id)
        {
            await _userService.DeleteAsync(id);
            return Ok();
        }
    }
}