﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Simbir.Market.Market.Attributes;
using Simbir.Market.Market.Controllers.DTO;
using Simbir.Market.Market.Services;
using Simbir.Market.Market.Services.DTO;
using System.ComponentModel.DataAnnotations;

namespace Simbir.Market.Market.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ProductsController : ControllerBase
    {
        private readonly IProductService _productService;

        public ProductsController(IProductService marketService)
        {
            _productService = marketService;
        }

        [HttpGet("{id:long}")]
        [ApiAuthorize]
        public async Task<IActionResult> GetProductByIdAsync([Required] long id)
        {
            var product = await _productService.GetProductByIdAsync(id);
            return Ok(product);
        }

        [HttpPost]
        [ApiAuthorize("Admin,Seller")]
        public async Task<IActionResult> CreateProduct([FromBody] ProductCreateRequest request)
        {
            var accessTokenHeader = HttpContext.Request.Headers["Authorization"].ToString();
            var accessToken = accessTokenHeader.Replace("Bearer ", "");

            var productId = await _productService.CreateProductAsync(request, accessToken);

            return Ok(productId);
        }

        [HttpPut("{id:long}")]
        [ApiAuthorize("Admin,Seller")]
        public async Task<IActionResult> UpdateProduct([Required] long id, [FromBody] ProductUpdateRequest request)
        {
            var accessTokenHeader = HttpContext.Request.Headers["Authorization"].ToString();
            var accessToken = accessTokenHeader.Replace("Bearer ", "");

            await _productService.UpdateProductAsync(id, request, accessToken);
            return Ok();
        }

        [HttpDelete("{id:long}")]
        [ApiAuthorize("Admin,Seller")]
        public async Task<IActionResult> DeleteProductByIdAsync([Required] long id)
        {
            var accessTokenHeader = HttpContext.Request.Headers["Authorization"].ToString();
            var accessToken = accessTokenHeader.Replace("Bearer ", "");

            await _productService.DeleteProductByIdAsync(id, accessToken);

            return Ok();
        }

        [HttpPost("{id:long}/Increase")]
        [ApiAuthorize("Admin,Seller")]
        public async Task<IActionResult> IncreaseProductAmountByIdAsync([Required] long id, [FromBody] ProductAmountRequest request)
        {
            var accessTokenHeader = HttpContext.Request.Headers["Authorization"].ToString();
            var accessToken = accessTokenHeader.Replace("Bearer ", "");

            await _productService.IncreaseProductAmountByIdAsync(id, request, accessToken);
            return Ok();
        }

        [HttpPost("{id:long}/Decrease")]
        [ApiAuthorize("Admin,Seller")]
        public async Task<IActionResult> DecreaseProductAmountByIdAsync([Required] long id, [FromBody] ProductAmountRequest request)
        {
            var accessTokenHeader = HttpContext.Request.Headers["Authorization"].ToString();
            var accessToken = accessTokenHeader.Replace("Bearer ", "");

            await _productService.DecreaseProductAmountByIdAsync(id, request, accessToken);
            return Ok();
        }

        [HttpGet]
        [ApiAuthorize]
        public async Task<IActionResult> GetAll([FromQuery] string? name, [FromQuery] List<string>? tags)
        {
            var filter = new ProductListFilterDTO
            {
                Name = name,
                Tags = tags
            };

            var response = await _productService.GetAllProductsAsync(filter);

            return Ok(response);
        }
    }
}