﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Options;
using Simbir.Market.Market.Configurations;
using Simbir.Market.Market.Controllers.DTO;
using Simbir.Market.Market.Data;
using Simbir.Market.Market.Exceptions;
using Simbir.Market.Market.Models;
using Simbir.Market.Market.Services.DTO;

namespace Simbir.Market.Market.Services
{
    public class ProductService : IProductService
    {
        private readonly DataContext _context;
        private readonly ExternalClientService _externalClientService;
        private readonly ExternalServiceBaseUrlConfig _externalServicesConfig;
        private readonly IMapper _mapper;

        public ProductService(DataContext context, ExternalClientService externalClientService, IOptions<ExternalServiceBaseUrlConfig> config, IMapper mapper)
        {
            _context = context;
            _externalClientService = externalClientService;
            _externalServicesConfig = config.Value;
            _mapper = mapper;
        }

        public async Task IncreaseProductAmountByIdAsync(long id, ProductAmountRequest request, string accessToken)
        {
            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            var product =
                await _context.Products.Where(p => p.Id == id && !p.IsDeleted)
                                       .FirstOrDefaultAsync()
                                       ?? throw new NotFoundException($"Товар с ID {id} не найдена.");

            if (!meInfo.Roles.Any(r => r == "Admin") && meInfo.Id != product.UserId)
            {
                throw new NotFoundException($"Товар с ID {id} не найдена.");
            }

            product.Amount += request.Amount;

            await _context.SaveChangesAsync();
        }

        public async Task DecreaseProductAmountByIdAsync(long id, ProductAmountRequest request, string accessToken)
        {
            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            var product =
                await _context.Products.Where(p => p.Id == id && !p.IsDeleted)
                                       .FirstOrDefaultAsync()
                                       ?? throw new NotFoundException($"Товар с ID {id} не найдена.");

            if (!meInfo.Roles.Any(r => r == "Admin") && meInfo.Id != product.UserId)
            {
                throw new NotFoundException($"Товар с ID {id} не найдена.");
            }

            if (product.Amount - request.Amount < 0)
            {
                throw new ApiException($"Не хватает товара на складе. Текущее количество: {product.Amount}");
            }

            product.Amount -= request.Amount;

            await _context.SaveChangesAsync();
        }

        public async Task<ProductDetailedInfoDTO> GetProductByIdAsync(long id)
        {
            var product = await
                (
                    from p in _context.Products
                    where !p.IsDeleted && p.Id == id
                    select new ProductDetailedInfoDTO
                    {
                        Id = p.Id,
                        Name = p.Name,
                        Description = p.Description,
                        Amount = p.Amount,
                        Price = p.Price,
                        TagList =
                            (
                                from t in _context.Tags
                                where t.ProductId == p.Id
                                select new TagInfoDTO
                                {
                                    Id = t.Id,
                                    Name = t.Name,
                                }
                            ).ToList()
                    }
                ).FirstOrDefaultAsync()
                ?? throw new NotFoundException($"Товар с ID {id} не найдена.");

            return product;
        }

        public async Task<long> CreateProductAsync(ProductCreateRequest request, string accessToken)
        {
            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            if (!meInfo.Roles.Any(r => r == "Admin" || r == "Seller"))
            {
                throw new ApiException("Действие запрещено.");
            }

            var productExists = _context.Products.Any(p => p.Name == request.Name && p.UserId == meInfo.Id && p.IsDeleted == false);

            if (productExists)
            {
                throw new ApiException("Товар с таким названием уже существует.");
            }

            var product = _mapper.Map<Product>(request);

            product.UserId = meInfo.Id;

            var tags =
                request.Tags.Select(r => new Tag
                {
                    Name = r
                }).ToList();

            using var tr = _context.Database.BeginTransaction();

            await _context.Products.AddAsync(product);

            await _context.SaveChangesAsync();

            foreach (var tag in tags)
            {
                tag.ProductId = product.Id;
            }

            await _context.Tags.AddRangeAsync(tags);
            await _context.SaveChangesAsync();

            await tr.CommitAsync();

            return product.Id;
        }

        public async Task UpdateProductAsync(long id, ProductUpdateRequest request, string accessToken)
        {
            var product =
                await _context.Products.Where(h => h.Id == id && !h.IsDeleted)
                                       .FirstOrDefaultAsync()
                                       ?? throw new NotFoundException($"Товар с ID {id} не найдена.");

            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            if (!meInfo.Roles.Any(r => r == "Admin") && meInfo.Id != product.UserId)
            {
                throw new NotFoundException($"Товар с ID {id} не найдена.");
            }

            var productExists = _context.Products.Any(p => p.Id != id && p.Name == request.Name && p.UserId == meInfo.Id && p.IsDeleted == false);

            if (productExists)
            {
                throw new ApiException("Товар с таким названием уже существует.");
            }

            _mapper.Map(request, product);

            var currentTags = await _context.Tags
                .Where(r => r.ProductId == id)
                .ToListAsync();

            var tagsToDelete = currentTags
                .Where(r => !request.Tags.Contains(r.Name))
                .ToList();

            var tagsToAdd = request.Tags
                .Where(name => !currentTags.Any(r => r.Name == name))
                .Select(name => new Tag
                {
                    ProductId = id,
                    Name = name
                })
                .ToList();

            _context.Tags.RemoveRange(tagsToDelete);

            await _context.Tags.AddRangeAsync(tagsToAdd);

            await _context.SaveChangesAsync();
        }

        public async Task DeleteProductByIdAsync(long id, string accessToken)
        {
            var product = await _context.Products
                .FirstOrDefaultAsync(p => p.Id == id && !p.IsDeleted)
                ?? throw new NotFoundException($"Товар с ID {id} не найдена.");

            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            if (!meInfo.Roles.Any(r => r == "Admin" || r == "Seller") && meInfo.Id != product.UserId)
            {
                throw new NotFoundException($"Товар с ID {id} не найдена.");
            }

            product.IsDeleted = true;

            await _context.SaveChangesAsync();
        }

        public async Task<List<ProductInformationDTO>> GetAllProductsAsync(ProductListFilterDTO filter)
        {
            var productsQuery = _context.Products.Where(p => !p.IsDeleted).AsQueryable();

            if (!string.IsNullOrEmpty(filter.Name))
            {
                productsQuery = productsQuery.Where(u => EF.Functions.Like(u.Name, "%" + filter.Name.ToLower() + "%"));
            }

            if (filter.Tags != null && filter.Tags.Count != 0)
            {
                var availableProduct =
                    from p in productsQuery
                    join t in _context.Tags.Where(t => filter.Tags.Contains(t.Name)) on p.Id equals t.ProductId
                    group new { ProductId = p.Id, TagId = t.Id } by p.Id into g
                    where g.GroupBy(g => g.ProductId).Count() == filter.Tags.Count
                    select g.Key;

                productsQuery =
                    from p in productsQuery
                    where availableProduct.Contains(p.Id)
                    select p;
            }

            var products = await
                (
                    from p in productsQuery
                    select new ProductInformationDTO
                    {
                        Id = p.Id,
                        Name = p.Name,
                        Description = p.Description,
                        Price = p.Price,
                        Amount = p.Amount,
                        Tags = _context.Tags.Where(t => t.ProductId == p.Id).Select(t => t.Name).ToList()
                    }
                ).ToListAsync();

            return products;
        }
    }
}
