﻿using AutoMapper;
using AutoMapper.QueryableExtensions;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Options;
using Simbir.Market.Purchase.Configurations;
using Simbir.Market.Purchase.Controllers.DTO;
using Simbir.Market.Purchase.Data;
using Simbir.Market.Purchase.Exceptions;
using Simbir.Market.Purchase.Models;
using Simbir.Market.Purchase.Services.DTO;

namespace Simbir.Market.Purchase.Services
{
    public class PurchaseService : IPurchaseService
    {
        private readonly DataContext _context;
        private readonly ExternalClientService _externalClientService;
        private readonly ExternalServiceBaseUrlConfig _externalServicesConfig;
        private readonly IMapper _mapper;

        public PurchaseService(DataContext context, ExternalClientService externalClientService, IOptions<ExternalServiceBaseUrlConfig> config, IMapper mapper)
        {
            _context = context;
            _externalClientService = externalClientService;
            _externalServicesConfig = config.Value;
            _mapper = mapper;
        }

        public async Task CancelPurchaseAsync(long id, string accessToken)
        {
            var purchase =
                await _context.Purchases.FirstOrDefaultAsync(p => p.Id == id && p.Status == PurchaseStatus.AwaitPayment)
                ?? throw new NotFoundException("Покупка не найдена.");

            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            if (!meInfo.Roles.Contains("Admin") && purchase.UserId != meInfo.Id)
            {
                throw new NotFoundException("Покупка не найдена.");
            }

            var increaseProductUrl = $"{_externalServicesConfig.MarketService}/api/Products/{purchase.ProductId}/Increase";

            var increaseProductBody = new
            {
                purchase.Amount
            };

            _ = await _externalClientService.SendRequestAsync<object>(accessToken, increaseProductUrl, HttpMethod.Post, increaseProductBody)
                ?? throw new ApiException("Ошибка при попытке вернуть товары на склад.");

            purchase.Status = PurchaseStatus.Cancelled;

            await _context.SaveChangesAsync();
        }

        public async Task ClosePurchaseAsync(long paymentId)
        {
            var purchase = await _context.Purchases.FirstOrDefaultAsync(p => p.PaymentId == paymentId && p.Status == PurchaseStatus.AwaitPayment);

            if (purchase == null)
            {
                return;
            }

            purchase.Status = PurchaseStatus.Completed;

            _context.Purchases.Update(purchase);
            await _context.SaveChangesAsync();
        }

        public async Task<long> CreatePurchaseAsync(PurchaseCreateRequest request, string accessToken)
        {
            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            var productInfoUrl = $"{_externalServicesConfig.MarketService}/api/Products/{request.ProductId}";

            var productInfo = await _externalClientService
                .SendRequestAsync<ProductDetailedInfoResponse>(accessToken, productInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Ошибка при попытке получения информации о товаре.");

            if (productInfo.Amount < request.Amount)
            {
                throw new ApiException($"Недостаточно товара на складе. Текущее количество: {productInfo.Amount}");
            }

            var decreaseProductUrl = $"{_externalServicesConfig.MarketService}/api/Products/{request.ProductId}/Decrease";

            var decreaseProductBody = new
            {
                request.Amount
            };

            await _externalClientService.SendRequestAsync<object>(accessToken, decreaseProductUrl, HttpMethod.Post, decreaseProductBody);

            var purchase = _mapper.Map<PurchaseEntity>(request);

            purchase.UserId = meInfo.Id;
            purchase.Status = PurchaseStatus.Created;
            purchase.TotalCost = purchase.Amount * productInfo.Price;

            await _context.AddAsync(purchase);
            await _context.SaveChangesAsync();

            var createPaymentUrl = $"{_externalServicesConfig.PaymentService}/api/Payment";

            var payment = new CreatePaymentRequest
            {
                PurchaseId = purchase.Id,
                Price = purchase.TotalCost
            };

            var paymentInfo = await _externalClientService
                .SendRequestAsync<CreatePaymentResponse>(accessToken, createPaymentUrl, HttpMethod.Post, payment)
                ?? throw new ApiException("Ошибка при попытке создания оплаты.");

            purchase.PaymentId = paymentInfo.PaymentId;
            purchase.Status = PurchaseStatus.AwaitPayment;

            _context.Purchases.Update(purchase);

            await _context.SaveChangesAsync();

            return paymentInfo.PaymentId;
        }

        public async Task<PurchaseHistoryInfoListDTO> GetPurchaseHistoryByUserIdAsync(long userId, int from, int count, string accessToken)
        {
            var purchaseQuery = _context.Purchases.Where(p => p.UserId == userId);
            var totalCount = purchaseQuery.Count();

            var purchaseList =
                await purchaseQuery.Skip(from)
                                   .Take(count)
                                   .ProjectTo<PurchaseHistoryInfoDTO>(_mapper.ConfigurationProvider)
                                   .ToListAsync();

            var result = new PurchaseHistoryInfoListDTO
            {
                Purchases = purchaseList,
                TotalCount = totalCount
            };

            return result;
        }

        public async Task<string> GetPurchaseStatusAsync(long id, string accessToken)
        {
            var purchase =
                await _context.Purchases.FirstOrDefaultAsync(p => p.Id == id)
                ?? throw new NotFoundException("Покупка не найдена.");

            var meInfoUrl = $"{_externalServicesConfig.AccountService}/api/Accounts/Me";

            var meInfo = await _externalClientService
                .SendRequestAsync<AccountMeInfoResponse>(accessToken, meInfoUrl, HttpMethod.Get, null)
                ?? throw new ApiException("Действие запрещено.");

            if (!meInfo.Roles.Contains("Admin") && purchase.UserId != meInfo.Id)
            {
                throw new NotFoundException("Покупка не найдена.");
            }

            return Enum.GetName(typeof(PurchaseStatus), purchase.Status);
        }
    }
}
